@@ -1,6 +1,9 @@
 use std::collections::HashMap;
+use std::env::current_dir;
 use std::fs::File;
 use std::io::{Read, Write};
+use std::path::PathBuf;
+use std::str::FromStr;
 
 use base64::{decode, encode};
 use proc_macro2::{Ident, Span};
@@ -23,7 +26,7 @@ fn parse_html_str(html: &str, fn_name: &str, ignore: &mut Vec<String>) -> proc_m
     for x in datas {
         if x.tag.eq("mapper") {
             for x in x.childs {
-                match x.attributes.get("id") {
+                match x.attrs.get("id") {
                     Some(id) => {
                         if id.eq(fn_name) {
                             return parse_html_node(vec![x], ignore);
@@ -48,7 +51,7 @@ fn find_element(id: &str, htmls: &Vec<Element>) -> Option<Element> {
                 return find;
             }
         }
-        match x.attributes.get("id") {
+        match x.attrs.get("id") {
             None => {}
             Some(id) => {
                 if id.eq(id) {
@@ -65,10 +68,10 @@ fn include_replace(htmls: Vec<Element>, sql_map: &mut HashMap<String, Vec<Elemen
     for mut x in htmls {
         match x.tag.as_str() {
             "sql" => {
-                sql_map.insert(x.attributes.get("id").expect("[rbatis] <sql> element must have id!").clone(), x.childs.clone());
+                sql_map.insert(x.attrs.get("id").expect("[rbatis] <sql> element must have id!").clone(), x.childs.clone());
             }
             "include" => {
-                let refid = x.attributes.get("refid").expect("[rbatis] <include> element must have refid!").clone();
+                let refid = x.attrs.get("refid").expect("[rbatis] <include> element must have refid!").clone();
                 let data_url = Url::parse(format!("url://{}", refid).as_str()).expect("[rbatis] parse include url fail!");
                 let mut find_file = false;
                 for (k, v) in data_url.query_pairs() {
@@ -99,7 +102,7 @@ fn include_replace(htmls: Vec<Element>, sql_map: &mut HashMap<String, Vec<Elemen
                 }
             }
             _ => {
-                match x.attributes.get("id") {
+                match x.attrs.get("id") {
                     None => {}
                     Some(id) => {
                         if !id.is_empty() {
@@ -129,9 +132,9 @@ fn include_replace(htmls: Vec<Element>, sql_map: &mut HashMap<String, Vec<Elemen
 
 fn parse_html_node(htmls: Vec<Element>, ignore: &mut Vec<String>) -> proc_macro2::TokenStream {
     #[cfg(feature = "debug_mode")]
-        {
-            println!("load html:{:#?}", htmls);
-        }
+    {
+        println!("load html:{:#?}", htmls);
+    }
     let mut methods = quote!();
     let fn_impl = parse(&htmls, &mut methods, "", ignore);
     let token = quote! {
@@ -213,7 +216,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                 }
             }
             "if" => {
-                let test_value = x.attributes.get("test").expect(&format!("{} element must be have test field!", x.tag));
+                let test_value = x.attrs.get("test").expect(&format!("{} element must be have test field!", x.tag));
                 let mut if_tag_body = quote! {};
                 if x.childs.len() != 0 {
                     if_tag_body = parse(&x.childs, methods, block_name, ignore);
@@ -222,15 +225,15 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
             }
             "trim" => {
                 let empty_string = String::new();
-                let prefix = x.attributes.get("prefix").unwrap_or(&empty_string).to_string();
-                let suffix = x.attributes.get("suffix").unwrap_or(&empty_string).to_string();
-                let prefixOverrides = x.attributes.get("prefixOverrides").unwrap_or(&empty_string).to_string();
-                let suffixOverrides = x.attributes.get("suffixOverrides").unwrap_or(&empty_string).to_string();
+                let prefix = x.attrs.get("prefix").unwrap_or(&empty_string).to_string();
+                let suffix = x.attrs.get("suffix").unwrap_or(&empty_string).to_string();
+                let prefixOverrides = x.attrs.get("prefixOverrides").unwrap_or(&empty_string).to_string();
+                let suffixOverrides = x.attrs.get("suffixOverrides").unwrap_or(&empty_string).to_string();
                 impl_trim(&prefix, &suffix, &prefixOverrides, &suffixOverrides, x, &mut body, arg, methods, &format!("{}:{}", block_name, "trim"), ignore);
             }
             "bind" => {
-                let name = x.attributes.get("name").expect("<bind> must be have name!").to_string();
-                let value = x.attributes.get("value").expect("<bind> element must be have value!").to_string();
+                let name = x.attrs.get("name").expect("<bind> must be have name!").to_string();
+                let value = x.attrs.get("value").expect("<bind> element must be have value!").to_string();
 
                 let name_expr = parse_expr(&name);
                 let method_impl = crate::func::impl_fn(&body.to_string(), "", &format!("\"{}\"", value), false, true, ignore);
@@ -259,7 +262,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                         panic!("choose node's childs must be when node and otherwise node!");
                     }
                     if x.tag.eq("when") {
-                        let test_value = x.attributes.get("test").expect(&format!("{} element must be have test field!", x.tag));
+                        let test_value = x.attrs.get("test").expect(&format!("{} element must be have test field!", x.tag));
                         let mut if_tag_body = quote! {};
                         if x.childs.len() != 0 {
                             if_tag_body = parse(&x.childs, methods, block_name, ignore);
@@ -290,17 +293,17 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                 let def_item = "item".to_string();
                 let def_index = "index".to_string();
 
-                let collection = x.attributes.get("collection").unwrap_or(&empty_string).to_string();
-                let mut item = x.attributes.get("item").unwrap_or(&def_item).to_string();
-                let mut findex = x.attributes.get("index").unwrap_or(&def_index).to_string();
-                let open = x.attributes.get("open").unwrap_or(&empty_string).to_string();
-                let close = x.attributes.get("close").unwrap_or(&empty_string).to_string();
-                let separator = x.attributes.get("separator").unwrap_or(&empty_string).to_string();
+                let collection = x.attrs.get("collection").unwrap_or(&empty_string).to_string();
+                let mut item = x.attrs.get("item").unwrap_or(&def_item).to_string();
+                let mut findex = x.attrs.get("index").unwrap_or(&def_index).to_string();
+                let open = x.attrs.get("open").unwrap_or(&empty_string).to_string();
+                let close = x.attrs.get("close").unwrap_or(&empty_string).to_string();
+                let separator = x.attrs.get("separator").unwrap_or(&empty_string).to_string();
 
-                if item.is_empty(){
+                if item.is_empty() {
                     item = def_item;
                 }
-                if findex.is_empty(){
+                if findex.is_empty() {
                     findex = def_index;
                 }
                 let mut ignores = ignore.clone();
@@ -394,7 +397,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
             }
 
             "select" => {
-                let id = x.attributes.get("id").expect("<select> element must be have id!");
+                let id = x.attrs.get("id").expect("<select> element must be have id!");
                 let method_name = Ident::new(id, Span::call_site());
                 let child_body = parse(&x.childs, methods, "select", ignore);
                 let select = quote! {
@@ -413,7 +416,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                         };
             }
             "update" => {
-                let id = x.attributes.get("id").expect("<update> element must be have id!");
+                let id = x.attrs.get("id").expect("<update> element must be have id!");
                 let method_name = Ident::new(id, Span::call_site());
                 let child_body = parse(&x.childs, methods, "select", ignore);
                 let select = quote! {
@@ -433,7 +436,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                         };
             }
             "insert" => {
-                let id = x.attributes.get("id").expect("<insert> element must be have id!");
+                let id = x.attrs.get("id").expect("<insert> element must be have id!");
                 let method_name = Ident::new(id, Span::call_site());
                 let child_body = parse(&x.childs, methods, "select", ignore);
                 let select = quote! {
@@ -453,13 +456,13 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                         };
             }
             "delete" => {
-                let id = x.attributes.get("id").expect("<delete> element must be have id!");
+                let id = x.attrs.get("id").expect("<delete> element must be have id!");
                 let method_name = Ident::new(id, Span::call_site());
                 let child_body = parse(&x.childs, methods, "select", ignore);
                 let select = quote! {
                             pub fn #method_name (arg:&rbson::Bson, _tag: char) -> (String,Vec<rbson::Bson>) {
                                use rbatis_sql::ops::AsProxy;
-                               
+
                                let mut sql = String::with_capacity(1000);
                                let mut args = Vec::with_capacity(20);
                                #child_body
@@ -482,10 +485,10 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
 
 
 fn impl_println(x: &Element, body: &mut proc_macro2::TokenStream, ignore: &mut Vec<String>) {
-    let value = x.attributes.get("value").expect(&format!("{} element must be have value field!", x.tag));
+    let value = x.attrs.get("value").expect(&format!("{} element must be have value field!", x.tag));
     let method_name = impl_method(value, body, ignore);
     let mut format = String::new();
-    if let Some(s) = x.attributes.get("format") {
+    if let Some(s) = x.attrs.get("format") {
         format = s.to_string();
     }
     if format.is_empty() {
@@ -525,23 +528,21 @@ fn impl_method(test_value: &str, body: &mut proc_macro2::TokenStream, ignore: &m
 fn impl_if(test_value: &str, if_tag_body: proc_macro2::TokenStream, body: &mut proc_macro2::TokenStream, methods: &mut proc_macro2::TokenStream, appends: proc_macro2::TokenStream, block_name: &str, ignore: &mut Vec<String>) {
     let method_name = impl_method(test_value, body, ignore);
     *body = quote! {
-                              #body
-                              if #method_name {
-                                   sql.push_str(" ");
-                                   #if_tag_body
-                                   sql.push_str(" ");
-                                   #appends
-                              }
-                          };
+                  #body
+                  if #method_name {
+                     #if_tag_body
+                     #appends
+                  }
+            };
 }
 
 fn impl_otherwise(child_body: proc_macro2::TokenStream, body: &mut proc_macro2::TokenStream, methods: &mut proc_macro2::TokenStream, block_name: &str, ignore: &mut Vec<String>) {
     *body = quote!(
-                        #body
-                        sql.push_str(" ");
-                        #child_body
-                        sql.push_str(" ");
-                       );
+                   #body
+                   sql.push_str(" ");
+                   #child_body
+                   sql.push_str(" ");
+            );
 }
 
 
@@ -585,16 +586,19 @@ fn impl_trim(prefix: &str, suffix: &str, prefixOverrides: &str, suffixOverrides:
 }
 
 pub fn impl_fn_html(m: &ItemFn, args: &AttributeArgs) -> TokenStream {
+    let current_dir = current_dir().unwrap();
     let fn_name = m.sig.ident.to_string();
     let mut file_name = args.get(0).to_token_stream().to_string();
     if file_name.ne("\"\"") && file_name.starts_with("\"") && file_name.ends_with("\"") {
         file_name = file_name[1..file_name.len() - 1].to_string();
     }
-    let t;
+    let mut t;
     #[cfg(feature = "debug_mode")]
-        {
-            println!("try open file:{}", file_name);
-        }
+    {
+        println!("try open file:{}", file_name);
+    }
+    let file_name_path_buf = PathBuf::from_str(&file_name).unwrap();
+
     let mut data = String::new();
     let mut f = File::open(file_name.as_str()).expect(&format!("File:\"{}\" does not exist", file_name));
     f.read_to_string(&mut data);
@@ -610,6 +614,16 @@ pub fn impl_fn_html(m: &ItemFn, args: &AttributeArgs) -> TokenStream {
         };
     }
     t = parse_html_str(&data, &fn_name, &mut vec![]);
+
+    #[cfg(feature = "debug_mode")]
+    {
+        let id = Ident::new(&format!("_include_{}", fn_name), Span::call_site());
+        let mut html_file_name = file_name.clone();
+        if !file_name_path_buf.is_absolute() {
+            html_file_name = format!("{}/{}", current_dir.to_str().unwrap_or_default(), file_name);
+        }
+        t = quote! {#t fn #id() {let _ = include_str!(#html_file_name);}};
+    }
     return t.into();
 }
 
@@ -621,9 +635,9 @@ pub fn impl_fn_py(m: &ItemFn, args: &AttributeArgs) -> TokenStream {
     }
     let t;
     #[cfg(feature = "debug_mode")]
-        {
-            println!("py_sql:{}", data);
-        }
+    {
+        println!("py_sql:{}", data);
+    }
     let mut format_char = '?';
     if args.len() > 1 {
         for x in args.get(1).to_token_stream().to_string().chars() {
@@ -636,9 +650,9 @@ pub fn impl_fn_py(m: &ItemFn, args: &AttributeArgs) -> TokenStream {
     let nodes = NodeType::parse(&data).expect("[rbatis] parse py_sql fail!");
     let htmls = crate::py_sql::to_html(&nodes, data.starts_with("select") || data.starts_with(" select"), &fn_name);
     #[cfg(feature = "debug_mode")]
-        {
-            println!("html:{}", htmls);
-        }
+    {
+        println!("html:{}", htmls);
+    }
     t = parse_html_str(&htmls, &fn_name, &mut vec![]);
     return t.into();
 }
